@@ -35,8 +35,7 @@ my %fields = (
     ACCESSLOG        => [ qw/USERID LOGDATE/ ],
 
     ANTIVIRUS        => [ qw/COMPANY ENABLED GUID NAME UPTODATE VERSION
-                             EXPIRATION BASE_CREATION BASE_VERSION
-                             32ENGINE_VERSION 64ENGINE_VERSION/ ],
+                             EXPIRATION BASE_CREATION BASE_VERSION/ ],
     BATTERIES        => [ qw/CAPACITY CHEMISTRY DATE NAME SERIAL MANUFACTURER
                              VOLTAGE/ ],
     CONTROLLERS      => [ qw/CAPTION DRIVER NAME MANUFACTURER PCICLASS VENDORID
@@ -45,7 +44,8 @@ my %fields = (
                              SERIAL STEPPING FAMILYNAME FAMILYNUMBER MODEL
                              SPEED ID EXTERNAL_CLOCK ARCH CORECOUNT/ ],
     DRIVES           => [ qw/CREATEDATE DESCRIPTION FREE FILESYSTEM LABEL
-                             LETTER SERIAL SYSTEMDRIVE TOTAL TYPE VOLUMN/ ],
+                             LETTER SERIAL SYSTEMDRIVE TOTAL TYPE VOLUMN
+                             ENCRYPT_NAME ENCRYPT_ALGO ENCRYPT_STATUS ENCRYPT_TYPE/ ],
     ENVS             => [ qw/KEY VAL/ ],
     INPUTS           => [ qw/NAME MANUFACTURER CAPTION DESCRIPTION INTERFACE
                              LAYOUT POINTINGTYPE TYPE/ ],
@@ -90,7 +90,8 @@ my %fields = (
     SOUNDS           => [ qw/CAPTION DESCRIPTION MANUFACTURER NAME/ ],
     STORAGES         => [ qw/DESCRIPTION DISKSIZE INTERFACE MANUFACTURER MODEL
                             NAME TYPE SERIAL SERIALNUMBER FIRMWARE SCSI_COID
-                            SCSI_CHID SCSI_UNID SCSI_LUN WWN/ ],
+                            SCSI_CHID SCSI_UNID SCSI_LUN WWN
+                            ENCRYPT_NAME ENCRYPT_ALGO ENCRYPT_STATUS ENCRYPT_TYPE/ ],
     VIDEOS           => [ qw/CHIPSET MEMORY NAME RESOLUTION PCISLOT PCIID/ ],
     USBDEVICES       => [ qw/VENDORID PRODUCTID MANUFACTURER CAPTION SERIAL
                             CLASS SUBCLASS NAME/ ],
