@@ -5,10 +5,14 @@ use FusionInventory::Agent::Tools;
 use strict;
 use warnings;
 
+use parent 'FusionInventory::Agent::Task::Inventory::Module';
+
 use English qw(-no_match_vars);
 
 sub isEnabled {
-    canRun('lspv');
+    my (%params) = @_;
+    return 0 if $params{no_category}->{lvm};
+    return canRun('lspv');
 }
 
 sub doInventory {
@@ -83,7 +87,7 @@ sub _getLogicalVolume {
             $volume->{SEG_COUNT} = $1;
         }
         if ($line =~ /^TYPE:\s+(\S+)/) {
-            $volume->{ATTR} = "Type $1",
+            $volume->{ATTR} = "Type $1";
         }
     }
     close $handle;
