@@ -3,6 +3,8 @@ package FusionInventory::Agent::Task::Inventory::Generic::Screen;
 use strict;
 use warnings;
 
+use parent 'FusionInventory::Agent::Task::Inventory::Module';
+
 use English qw(-no_match_vars);
 use MIME::Base64;
 use UNIVERSAL::require;
@@ -17,6 +19,12 @@ sub isEnabled {
     return 1;
 }
 
+sub isEnabledForRemote {
+    my (%params) = @_;
+    return 0 if $params{no_category}->{monitor};
+    return 1;
+}
+
 sub doInventory {
     my (%params) = @_;
 
@@ -214,7 +222,7 @@ sub _getScreensFromUnix {
     if (canRun('get-edid')) {
         my $edid;
         foreach (1..5) { # Sometime get-edid return an empty string...
-            $edid = getFirstLine(command => 'get-edid');
+            $edid = getAllLines(command => 'get-edid');
             last if $edid;
         }
         $logger->debug_result(
