@@ -355,7 +355,7 @@ sub getFirstLine {
     my $result = <$handle>;
     close $handle;
 
-    chomp $result if $result;
+    chomp $result if defined $result;
     return $result;
 }
 
@@ -371,7 +371,7 @@ sub getLastLine {
     }
     close $handle;
 
-    chomp $result if $result;
+    chomp $result if defined $result;
     return $result;
 }
 
@@ -829,4 +829,4 @@ Return the content of a given file.
 
 =head2 isParamArrayAndFilled($hash, $paramName)
 
-Return if $hash has a key $paramName which value is a ARRAY ref not empty.
\ No newline at end of file
+Return if $hash has a key $paramName which value is a ARRAY ref not empty.
